/*  Jxspectrum - a spectrum analyzer.
    Copyright (C) 1997  Jani Frilander

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.*/

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "plot.h"

void DoFFT(PlotPoint *points);

LogPlot::LogPlot( QWidget *parent, const char *name )
   : QWidget( parent, name )
{
    Min_x = 0;
    Max_x = 0;
    Min_y = 0;
    Max_y = 0;
    Color_mode = 0;
    pointcount = 0;
    pixmap.resize(1,1);
    
    initMetaObject();
    timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), SLOT(timeout()) );
    timer->start( 40 );
    timer_interval=40;
    
    drawcolor = new QColor;
    gridcolor = new QColor;
    
    *drawcolor = QColor ( black );
    setBackgroundColor( white );
    *gridcolor = QColor ( 127,127,127 );
}

LogPlot::~LogPlot()
{
    if (pointcount)
      FreePoints();
    delete drawcolor;
    delete gridcolor;
    delete timer;
}

void LogPlot::StartTimer(int time)
{
    if (timer->isActive())
      timer->changeInterval( time );
    else
      timer->start( time );
    timer_interval=time;
}

void LogPlot::StopTimer()
{
    if (timer->isActive())
      timer->stop();
}

void LogPlot::ContinueTimer()
{
    if (timer->isActive())
      timer->changeInterval( timer_interval );
    else
      timer->start( timer_interval );
}

void LogPlot::SetLimits(double min_x,double max_x,double min_y,double max_y)
{
    Min_x = min_x;
    Max_x = max_x;
    Min_y = min_y;
    Max_y = max_y;
}

void LogPlot::SetParameters(char color_mode,char *unit_x,
			    char *unit_y,char *title)
{
    Color_mode = color_mode;

    Unit_x=unit_x;
    Unit_y=unit_y;
    Title=title;
    
    switch (Color_mode)
    {
     case CHART_LOOK_NORMAL:
	*drawcolor = QColor ( black );
	setBackgroundColor( white );
	*gridcolor = QColor ( 127,127,127 );
	break;
     case CHART_LOOK_REVERSE:
	*drawcolor = QColor ( white );
	setBackgroundColor( black );
	*gridcolor = QColor ( 127,127,127 );
	break;
     case CHART_LOOK_SCOPE:
	setBackgroundColor( black );
	*drawcolor = QColor ( 127,255,200 );
	*gridcolor = QColor ( 63,127,100 );
    }
}

void LogPlot::paintEvent( QPaintEvent * )
{
    UpdatePlot();
}

void LogPlot::UpdatePlot()
{
    int size_x,size_y;
    int current;
    double i;
    double scale_x;
    double scale_y;
    double offset_x;
    char buf[128];
    char a;
    double tmp=0;
    int tmp2=0;
    int tmp_x1,tmp_y1,tmp_x2,tmp_y2;
    int factor;
    int j;
    
    if(pixmap.size() != size())
      pixmap.resize( size() ); 
    
    pixmap.fill(backgroundColor());
    paint.begin(&pixmap);
    
    size_x=width()-70;
    size_y=height()-70;
    i=Min_x;
    offset_x=log10(i);
    current=0;
    scale_x=size_x/(log10(Max_x)-offset_x);
    scale_y=(int) ((double) size_y)/((double) (Max_y-Min_y));
    paint.setPen(*drawcolor);

    sprintf(buf,"%s",Unit_y);
    paint.drawText(pos_x-10,pos_y-10,buf);
    sprintf(buf,"%s",Title);
    paint.drawText(pos_x+20,0,size_x-40,pos_y,AlignCenter,buf);
    sprintf(buf,"%s",Unit_x);
    paint.drawText(pos_x+size_x+4,pos_y+size_y+3,buf);
    
    paint.setPen(*gridcolor);
    
    while (i<=(1.001*Max_x)) // 1.001 added for inaccuracy
    {	
	sprintf(buf,"%e",i);
	if((buf[0]=='1') || (buf[0]=='2') || (buf[0]=='5'))
	{
	    tmp2=5;
	    factor=(int) floor(log10(i));
	    switch (factor)
	    {
	     case -18:
	     case -17:
	     case -16:
		a='a';
		tmp=i*1e18;
		break;
	     case -15:
	     case -14:
	     case -13:
		a='f';
		tmp=i*1e15;
		break;
	     case -12:
	     case -11:
	     case -10:
		a='p';
		tmp=i*1e12;
		break;
	     case -9:
	     case -8:
	     case -7:
		a='n';
		tmp=i*1e9;
		break;
	     case -6:
	     case -5:
	     case -4:
		a='u';
		tmp=i*1e6;
		break;
	     case -3:
	     case -2:
	     case -1:
		a='m';
		tmp=i*1e3;
		break;
	     case 3:
	     case 4:
	     case 5:
		a='k';
		tmp=i/1e3;
		break;
	     case 6:
	     case 7:
	     case 8:
		a='M';
		tmp=i/1e6;
		break;
	     case 9:
	     case 10:
	     case 11:
		a='G';
		tmp=i/1e9;
		break;
	     case 12:
	     case 13:
	     case 14:
		a='T';
		tmp=i/1e12;
		break;
	     case 15:
	     case 16:
	     case 17:
		a='P';
		tmp=i/1e15;
		break;
	     case 18:
	     case 19:
	     case 20:
		a='E';
		tmp=i/1e18;
		break;
	     default:
		a='\0';
		tmp=i;
	    }
	    
	    if((factor <= 20) && (factor >= -18))
	    {
		sprintf(buf,"%.0f%c",tmp,a);
	    }
	    else
	    {
		sprintf(buf,"%.0e",i);
	    }
	    paint.setPen(*drawcolor);
	    paint.drawText(pos_x+current-5,pos_y+size_y+30,buf);
	    paint.setPen(*gridcolor);
	}
	else
	  tmp2=0;
	
	paint.drawLine(pos_x+current,pos_y,pos_x+current,pos_y+size_y-1+tmp2);
	
	i+=(pow(10,(floor(log10(i))+1e-10))); // 1e-10 added for inaccuracey
	current=(int) ((log10(i) - offset_x) *  scale_x);
    }
    
    /*ftime(times);
    printf("%d %ld.%d\n",2,times->time,times->millitm);*/
    
    current=0;
    i=Max_y;
    while (i>=Min_y)
    {
	paint.drawLine(pos_x-5,pos_y+current,pos_x+size_x-1,pos_y+current);
	sprintf(buf,"%.0f",i);
	paint.setPen(*drawcolor);
	paint.drawText(pos_x-35,pos_y+current+5,buf);
	paint.setPen(*gridcolor);
	i-=10;
	current=(int) ((Max_y-i)*scale_y);
    }
    
    paint.setPen(*drawcolor);
    
    tmp_x1 = (int) (pos_x+(log10(points[0].x)-offset_x)*scale_x);
    tmp_y1 = (int) (pos_y+((Max_y-points[0].y)*scale_y));    
    for (j=1;j<pointcount;j++)
    {
	tmp_x2 = (int) (pos_x+(log10(points[j].x)-offset_x)*scale_x);
	tmp_y2 = (int) (pos_y+((Max_y-points[j].y)*scale_y));
	if(((tmp_x1 >= pos_x) && (tmp_x1 <= (pos_x+size_x))) && 
	   ((tmp_y1 <= (pos_y+size_y)) && (tmp_y1 >= pos_y)) &&
	   ((tmp_x2 >= pos_x) && (tmp_x2 <= (pos_x+size_x))) &&
	   ((tmp_y2 <= (pos_y+size_y)) && (tmp_y2 >= pos_y)))
	paint.drawLine(tmp_x1,tmp_y1,tmp_x2,tmp_y2);
	tmp_x1=tmp_x2;
	tmp_y1=tmp_y2;
    }	
    
    paint.end();
    
    bitBlt( this, 0,0, &pixmap );
}

void LogPlot::AllocPoints(int count)
{
    if(pointcount)
      FreePoints();
    points = new PlotPoint[count];
    pointcount = count;
}

void LogPlot::FreePoints()
{
    delete points;
    pointcount = 0;
}

void LogPlot::SetPoint(int index,float x,float y)
{
    points[index].x=x;
    points[index].y=y;
}

void LogPlot::timeout()
{
    DoFFT(points);
    UpdatePlot();
}
